"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
exports.__esModule = true;
var conventional_recommended_version_1 = require("conventional-recommended-version");
var path_1 = require("path");
var child_process_1 = require("./lib/child-process");
var fs_1 = require("./lib/fs");
var log_1 = require("./lib/log");
var changelog = require('conventional-changelog');
var bumpVersion = function (directory, nextVersion) {
    return child_process_1.exec("npm version " + nextVersion + " --no-git-tag-version --force", { cwd: directory })["catch"](function (err) {
        log_1.bug("failed to run \"npm version\" with version \"" + nextVersion + "\"", err);
        process.exit(1);
    });
};
var tagExists = function (tagName) {
    return child_process_1.exec("git tag --list " + tagName).then(function (_a) {
        var stdout = _a.stdout;
        return stdout === tagName;
    });
};
var abortOnExistingTag = function (force, nextVersion, tagRelease) {
    return Promise.resolve().then(function () { return __awaiter(_this, void 0, void 0, function () {
        var _a;
        return __generator(this, function (_b) {
            switch (_b.label) {
                case 0:
                    _a = tagRelease && !force;
                    if (!_a) return [3 /*break*/, 2];
                    return [4 /*yield*/, tagExists(nextVersion)];
                case 1:
                    _a = (_b.sent());
                    _b.label = 2;
                case 2:
                    if (_a) {
                        log_1.error("the tag " + nextVersion + " already exists");
                        process.exit(1);
                    }
                    return [2 /*return*/];
            }
        });
    }); });
};
var abortOnUnchangedVersion = function (manifestPath, nextVersion) {
    return Promise.resolve().then(function () {
        var currentVersion = require(manifestPath).version;
        if (nextVersion === currentVersion) {
            log_1.error("the current version is already " + nextVersion);
            process.exit(1);
        }
    });
};
var commitChanges = function (directory, nextVersion, skipHooks) {
    var baseArgs = ['commit', '-m', "chore(release): " + nextVersion];
    var args = skipHooks ? baseArgs.concat('--no-verify') : baseArgs;
    return child_process_1.spawn('git', args, { cwd: directory });
};
var getVersion = function (directory, postfix) {
    return new Promise(function (resolve, reject) {
        return conventional_recommended_version_1["default"]({ directory: directory, postfix: postfix }, function (err, version) { return (err ? reject(err) : resolve(version)); });
    });
};
var stageChanges = function (directory) {
    return child_process_1.exec('git add . -A', { cwd: directory })["catch"](function (err) {
        log_1.bug('failed to stage changes for release commit', err);
        process.exit(1);
    });
};
var tagCommit = function (directory, force, nextVersion) {
    return child_process_1.exec("git tag " + nextVersion + (force ? ' --force' : ''), { cwd: directory });
};
var generateChangelog = function (changelogPath, directory) {
    return fs_1.unlink(changelogPath)
        .then(function () {
        return new Promise(function (resolve, reject) {
            var fileWrite$ = fs_1.createWriteStream(changelogPath);
            fileWrite$.on('finish', function (err) { return (err ? reject(err) : resolve()); });
            changelog({ path: directory, preset: 'angular', releaseCount: 0 }).pipe(fileWrite$);
        });
    })["catch"](function (err) {
        log_1.bug('failed to generate changelog', err);
        process.exit(1);
    });
};
var generateDependencyReport = function (dependencyLogPath, manifestPath) {
    return child_process_1.spawn('node', [require.resolve('package-json-to-readme'), '--no-footer', manifestPath])
        .then(function (stdout) { return fs_1.writeFile(dependencyLogPath, stdout, { encoding: 'utf8' }); })["catch"](function (err) {
        log_1.bug('failed to generate dependency report', err);
        process.exit(1);
    });
};
exports.commitRelease = function (_a) {
    var directory = _a.directory, _b = _a.force, force = _b === void 0 ? false : _b, _c = _a.postfix, postfix = _c === void 0 ? '' : _c, _d = _a.skipHooks, skipHooks = _d === void 0 ? false : _d, _e = _a.tagRelease, tagRelease = _e === void 0 ? true : _e, _f = _a.version, version = _f === void 0 ? '' : _f;
    return __awaiter(_this, void 0, void 0, function () {
        var nextVersion, _g, changelogPath, dependencyLogPath, manifestPath;
        return __generator(this, function (_h) {
            switch (_h.label) {
                case 0:
                    _g = version;
                    if (_g) return [3 /*break*/, 2];
                    return [4 /*yield*/, getVersion(directory, postfix)];
                case 1:
                    _g = (_h.sent());
                    _h.label = 2;
                case 2:
                    nextVersion = _g;
                    changelogPath = path_1.resolve(directory, 'CHANGELOG.md');
                    dependencyLogPath = path_1.resolve(directory, 'DEPENDENCIES.md');
                    manifestPath = path_1.resolve(directory, 'package.json');
                    return [4 /*yield*/, abortOnUnchangedVersion(manifestPath, nextVersion)];
                case 3:
                    _h.sent();
                    return [4 /*yield*/, abortOnExistingTag(force, nextVersion, tagRelease)];
                case 4:
                    _h.sent();
                    return [4 /*yield*/, bumpVersion(directory, nextVersion)];
                case 5:
                    _h.sent();
                    log_1.info("version: " + nextVersion);
                    return [4 /*yield*/, generateChangelog(changelogPath, directory)];
                case 6:
                    _h.sent();
                    log_1.info("changelog: " + changelogPath);
                    return [4 /*yield*/, generateDependencyReport(dependencyLogPath, manifestPath)];
                case 7:
                    _h.sent();
                    log_1.info("dependency report: " + dependencyLogPath);
                    return [4 /*yield*/, stageChanges(directory)];
                case 8:
                    _h.sent();
                    return [4 /*yield*/, commitChanges(directory, nextVersion, skipHooks)];
                case 9:
                    _h.sent();
                    log_1.info("release committed");
                    if (!tagRelease) return [3 /*break*/, 11];
                    return [4 /*yield*/, tagCommit(directory, force, nextVersion)];
                case 10:
                    _h.sent();
                    log_1.info("release tagged");
                    _h.label = 11;
                case 11:
                    log_1.success('complete');
                    return [2 /*return*/];
            }
        });
    });
};
